import os, sys, time

from twisted.application.internet import TimerService
from twisted.internet.defer import Deferred, inlineCallbacks, returnValue
from twisted.plugin import IPlugin
from zope.interface import implements

from jersey import log
from vtwt import cli


class WatchOptions(cli.Options):

    optParameters = [
            ["limit", "L", None,
                "The maximum number of messages to be displayed at once.", int],
            ["interval", "i", None, "Time between requests", int],
        ]

    optFlags = [
            ["long", "l", "Display in messages long format."],
        ]


    def parseArgs(self, watchee="home"):
        self["watchee"] = watchee

    def postOptions(self):
        log.debug("Going to watch {0[watchee]} ")


class Watcher(cli.Command):

    def execute(self):
        log.trace("Executing watcher.")

        self._lastPrintedId = None
        self._lastError = None
        if self.config["interval"]:
            svc = TimerService(self.config["interval"], self.showTimeline)
            svc.setServiceParent(self)

            # Since this runs ~forever, just return a Deferred that doesn't call
            # back.  A swift SIGINT will kill it.
            d = Deferred()

        else:
            # Print it once and exit
            d = self.showTimeline()

        return d


    @inlineCallbacks
    def showTimeline(self):
        try:
            log.debug(("Requesting {0.config[watchee]} timeline since "
                       "{0._lastPrintedId}").format(self))
            watchee = self.config["watchee"]
            params = dict()

            if self._lastPrintedId:
                params["since_id"] = self._lastPrintedId

            messages = yield self.vtwt.getTimeline(watchee, params)

            messages = self._limitMessages(messages, self.config["limit"])
            if messages:
                self.printMessages(messages)
                self._lastPrintedId = messages[-1].id

        except Exception, e:
            if repr(self._lastError) != repr(e):
                print >>sys.stderr, self.failWhale(e)
                self._lastError = e
        else:
            self._lastError = None


    @staticmethod
    def _limitMessages(messages, limit=None):
        if limit and limit < len(messages):
            log.trace("Limiting messages: {0}".format(", ".join(
                    m.id for m in messages[:-limit])))
            del messages[:-limit]
        return messages


    def printMessages(self, messages):
        screenNameWidth = max(len(msg.user.screen_name) for msg in messages)
        for msg in messages:
            self.printMessage(msg, screenNameWidth)

        if messages and self.config["interval"] and not self.config["long"]:
            self.printTimestamp()

    def printTimestamp(self):
        self._print("# {0}".format(time.ctime()))


    def printMessage(self, msg, screenNameWidth=None):
        if self.config["long"]:
            text = self.formatMsgLong(msg, screenNameWidth)
        else:
            text = self.formatMsgSimple(msg, screenNameWidth)
        self._print(text)
        return msg


class WatchLoader(cli.CommandFactory):
    implements(IPlugin)

    description = "Watch the twitter"
    name = "watch"
    shortcut = "w"
    options = WatchOptions
    command = Watcher


loader = WatchLoader()




import sys

from twisted.internet.defer import inlineCallbacks
from twisted.plugin import IPlugin
from zope.interface import implements

from jersey import log
from vtwt import cli


class FollowOptions(cli.Options):

    def parseArgs(self, *names):
        if not names:
            raise cli.UsageError("No one to follow ;(")
        self["friends"] = names


class Follower(cli.Command):

    @inlineCallbacks
    def execute(self):
        users = []
        for friend in self.config["friends"]:
            try:
                user = yield self.vtwt.follow(friend)
                self._printFollowee(user)

            except Exception, e:
                print >>sys.stderr, self.failWhale(e)



    def _printFollowee(self, user):
        print "{u.screen_name}".format(c=self.config, u=user)



class FollowLoader(cli.CommandFactory):
    implements(IPlugin)

    description = "Follow the given user"
    name = "follow"
    shortcut = "f"
    options = FollowOptions
    command = Follower


loader = FollowLoader()




import os, sys

from twisted.application.service import Service
from twisted.internet.defer import Deferred, inlineCallbacks, returnValue

from oauth import oauth

from jersey import log

from twittytwister.twitter import TwitterFeed

from vtwt.util import recodeText



class VtwtService(Service):

    oauthConsumer = oauth.OAuthConsumer("xkCemphqejlUZ9W11MUMsA",
            "QsnJx6SDhH6SjWNG95jcUR3FnKyFWFoOvDcf90E")

    baseUrl = "https://api.twitter.com"


    def __init__(self, accessToken):
        self.name = "vtwt:{0}".format(accessToken.key)
        self.accessToken = accessToken
        self._twt = self._buildTwitterClient(accessToken)


    def _buildTwitterClient(self, token):
        return TwitterFeed(consumer=self.oauthConsumer, token=token,
                base_url=self.baseUrl)


    @inlineCallbacks
    def getTimeline(self, user=None, params={}):
        """Get recent updates from a user's timeline.
        """
        log.trace("Getting {0} timeline".format(user or "home"))

        messages = []
        def addMessage(msg):
            msg.text = self._recodeText(msg.text)
            messages.insert(0, msg)

        if user in (None, "home"):
            yield self._twt.home_timeline(addMessage, params)
        else:
            yield self._twt.user_timeline(addMessage, user, params)

        returnValue(messages)


    def _recodeText(self, text):
        """Recode HTML entities refs; e.g. '&amp;' to '&'
        
        Work around buggy Twitter clients that mangle retweets such that
        &amp;lt;3 recodes to <3 (instead of &lt;3).
        """
        return recodeText(recodeText(text))


    def tweet(self, text):
        return self._twt.update(text)


    @inlineCallbacks
    def retweet(self, msgId):
        msgs = []
        yield self._twt.retweet(msgId, msgs.append)
        returnValue(msgs[0])


    @inlineCallbacks
    def follow(self, user):
        """Follow a user.  Return a dictionary representing the user.
        """
        users = []
        yield self._twt.follow_user(user, users.append)
        returnValue(users[0])


    @inlineCallbacks
    def unfollow(self, user):
        """Unfollow a user.  Return a dictionary representing the user.
        """
        users = []
        yield self._twt.unfollow_user(user, users.append)
        returnValue(users[0])


    def block(self, user):
        """Block a user.
        """
        users = []
        return self._twt.block(user)


    def unblock(self, user):
        """Unblock a user.
        """
        return self._twt.unblock(user)


    @inlineCallbacks
    def getFollowers(self, user=None):
        """Get a list of followers (optionally, for another user).
        """
        followers = []
        yield self._twt.list_followers(lambda f: followers.insert(0, f), user)
        returnValue(followers)


    @inlineCallbacks
    def getFollowees(self, user=None):
        """Get a list of users following 
        """
        followees = []
        yield self._twt.list_friends(lambda f: followees.insert(0, f), user)
        returnValue(followees)






import sys

from twisted.internet.defer import inlineCallbacks, gatherResults
from twisted.plugin import IPlugin
from twisted.python import usage
from zope.interface import implements

from jersey import log
from vtwt import cli


class UnBlockOptions(cli.Options):

    def parseArgs(self, *names):
        if not names:
            raise usage.error("No one to unblock ;(")
        self["unblockees"] = names


class UnBlocker(cli.Command):

    @inlineCallbacks
    def execute(self):
        for unblockee in self.config["unblockees"]:
            try:
                yield self._unblock(unblockee)

            except Exception, e:
                print >>sys.stderr, self.failWhale(e)



    @inlineCallbacks
    def _unblock(self, user):
        yield self.vtwt.unblock(user)
        print "{0}".format(user)



class UnBlockLoader(cli.CommandFactory):
    implements(IPlugin)

    description = "Un-block the given user"
    name = "unblock"
    shortcut = "B"
    options = UnBlockOptions
    command = UnBlocker


loader = UnBlockLoader()




import os, re, sys, time

from twisted.application.internet import TimerService
from twisted.internet import reactor
from twisted.internet.defer import Deferred, inlineCallbacks, returnValue
from twisted.plugin import IPlugin
from twisted.web import error as weberr, http
from zope.interface import implements

from jersey import log
from vtwt import cli


class GrepOptions(cli.Options):

    optFlags = [
            ["ignore-case", "i", "Ignore case"],
            ["count", "c", "Count the number of matches.  Implies 'quiet'."],
            #["exclude-retweets", "p", "Print the.  Implies 'quiet'."],
            ["long", "l", "Print messages in long format"],
            ["percentage", "p", "Print the.  Implies 'quiet'."],
            ["quiet", "q", "Be quiet.  Don't print results."],
        ]

    synopsis = "[options] regex [user]"

    def parseArgs(self, regex, grepee=None):
        try:
            self["regex"] = re.compile(regex)
        except re.error, e:
            raise cli.UsageError(str(e))
        self["grepee"] = grepee


    @property
    def grepee(self):
        return self["grepee"] or self.parent["user"]


    @property
    def summaryFormat(self):
        fmt = ""
        if self["count"]:
            fmt += "{count}"
            if self["percentage"]:
                fmt += "\t{total}"
        if self["percentage"]:
            if fmt:
                fmt += "\t"
            fmt += "{percentage:.2%}"
        return fmt



class Grepper(cli.Command):

    _count = str(200)
    sleepTime = 2

    @inlineCallbacks
    def execute(self):
        try:
            log.trace("Executing Grepper on {0.config.grepee}.".format(self))
            printMatches = not (self.config["count"] or
                    self.config["percentage"] or
                    self.config["quiet"])

            page = 1
            maxId = None
            messages = None
            messageCount = matchCount = 0
            while page == 1 or messages:
                try:
                    messages = yield self._getMessages(page, maxId)

                except weberr.Error, we:
                    if int(we.status) == http.BAD_GATEWAY:
                        print >>sys.stderr, self.failWhale(we)
                        d = Deferred()
                        reactor.callLater(self.sleepTime, d.callback, True)
                        yield d
                    else:
                        raise

                else:
                    for msg in messages:
                        messageCount += 1
                        log.trace("Message {0}".format(messageCount))
                        maxId = long(msg.id) - 1 
                        if self._grepMessage(msg):
                            matchCount += 1
                            log.trace("Match {0}".format(matchCount))
                            if printMatches:
                                self.printMessage(msg)
                    page += 1

            if self.config["percentage"] or self.config["count"]:
                self._printSummary(matchCount, messageCount)

            self.exitValue = 0 if matchCount > 0 else 1
        
        except Exception, e:
            print >>sys.stderr, self.failWhale(e)
            self.exitValue = os.EX_SOFTWARE

        returnValue(self.exitValue)


    @inlineCallbacks
    def _getMessages(self, page=1, maxId=None):
        params = dict(count=self._count)  #, page=str(page))
        log.debug("Page: {0}".format(page))

        if maxId:
            params["max_id"] = str(maxId)
            log.debug("Max ID: " + params["max_id"])

        msgs = yield self.vtwt.getTimeline(self.config.grepee, params)
        log.debug("Returned: " + str(len(msgs)))

        msgs.reverse()
        returnValue(msgs)


    def _grepMessage(self, msg):
        log.trace(self.formatMsgSimple(msg))
        return self.config["regex"].search(msg.text, self._grepFlags)

    @property
    def _grepFlags(self):
        flags = 0
        if self.config["ignore-case"]:
            flags |= re.IGNORECASE
        return flags


    def printMessage(self, msg, screenNameWidth=None):
        if self.config["long"]:
            text = self.formatMsgLong(msg, screenNameWidth)
        else:
            text = self.formatMsgSimple(msg, screenNameWidth)
        self._print(text)
        return msg


    def _printSummary(self, matchCount, totalCount):
        self._print(self.config.summaryFormat.format(
            count=matchCount, total=totalCount,
            percentage=float(matchCount) / totalCount))



class GrepLoader(cli.CommandFactory):
    implements(IPlugin)

    description = "Grep tweets"
    name = "grep"
    shortcut = "g"
    options = GrepOptions
    command = Grepper


loader = GrepLoader()




import sys

from twisted.internet.defer import Deferred, inlineCallbacks, returnValue
from twisted.plugin import IPlugin
from zope.interface import implements

from jersey import log
from vtwt import cli


class RetweetOptions(cli.Options):

    optFlags = [
            ["long", "l", "Display messages in long format."],
        ]

    def parseArgs(self, msgId):
        self["id"] = msgId


class Retweeter(cli.Command):

    @inlineCallbacks
    def execute(self):
        try:
            msg = yield self.vtwt.retweet(self.config["id"])
            self.printMessage(msg)

        except Exception, e:
            print >>sys.stderr, self.failWhale(e)
            self.exitValue = os.EX_SOFTWARE

    def printMessage(self, msg, screenNameWidth=None):
        if self.config["long"]:
            text = self.formatMsgLong(msg, screenNameWidth)
        else:
            text = self.formatMsgSimple(msg, screenNameWidth)
        self._print(text)
        return msg



class RetweetLoader(cli.CommandFactory):
    implements(IPlugin)

    description = "Retweet a message"
    name = "retweet"
    shortcut = "r"
    options = RetweetOptions
    command = Retweeter


loader = RetweetLoader()




import sys

from twisted.internet.defer import inlineCallbacks, returnValue
from twisted.plugin import IPlugin
from zope.interface import implements

from jersey import log
from vtwt import cli


class FollowingOptions(cli.Options):
    optFlags = [
            ["long", "l", "Print more user information."],
        ]


class Following(cli.Command):

    @inlineCallbacks
    def execute(self):
        try:
            friends = yield self.vtwt.getFollowees()
            self._printFriends(friends)

        except Exception, e:
            print >>sys.stderr, self.failWhale(e)
            raise SystemError(1)



    def _printFriends(self, friends):
        for friend in friends:
            self._printFriend(friend)

    def _printFriend(self, friend):
        if self.config["long"]:
            fmt = "{0.screen_name} ({0.name})"
        else:
            fmt = "{0.screen_name}"

        print fmt.format(friend)



class FollowingLoader(cli.CommandFactory):
    implements(IPlugin)

    description = "List friends"
    name = "following"
    shortcut = "F"
    options = FollowingOptions
    command = Following


loader = FollowingLoader()




import re
from htmlentitydefs import name2codepoint

from twisted.python.text import greedyWrap
from twisted.web.error import Error as WebError


# From http://wiki.python.org/moin/EscapingHtml

_HTMLENT_CODEPOINT_RE =  re.compile('&({0}|#\d+);'.format(
        '|'.join(name2codepoint.keys())))

def recodeText(text):
    """Parses things like &amp; and &#8020; into real characters."""
    def _entToUnichr(match):
        ent = match.group(1)
        try:
            if ent.startswith("#"):
                char = unichr(int(ent[1:]))
            else:
                char = unichr(name2codepoint[ent])
        except:
            char = match.group(0)

        return char

    return _HTMLENT_CODEPOINT_RE.sub(_entToUnichr, text)



def failWhale(error, columns=80):
    if isinstance(error, WebError):
        emsg = "{0.status} {0.message}".format(error)
    else:
        emsg = str(error)

    return whale(emsg, columns)


_whaleFmt = """\
   _{lines}__
|\\/{space} x \\
}}   {body}   |
|/\\{lines}__-/"""

_whalePaddingLen = 6


def whale(msg, columns=80):
    width = columns - _whalePaddingLen
    lines = []
    for line in msg.splitlines():
        lines.extend(greedyWrap(line, width))
    lineLength = max(map(len, lines))

    msg = "{0}|\n|{0}".format((_whalePaddingLen/2)*" ").join(
                map(lambda l: "{0:{1}}".format(l, lineLength),
                    lines))

    return _whaleFmt.format(
            space = " "*lineLength,
            lines = "_"*lineLength,
            length = lineLength,
            body = msg)





import os, sys

from twisted.internet.defer import inlineCallbacks
from twisted.plugin import IPlugin
from zope.interface import implements

from jersey import log
from vtwt import cli


class UnFollowOptions(cli.Options):

    def parseArgs(self, *names):
        if not names:
            raise usage.error("No one to unfollow ;(")
        self["losers"] = names


class UnFollower(cli.Command):

    @inlineCallbacks
    def execute(self):
        users = []
        for loser in self.config["losers"]:
            try:
                user = yield self.vtwt.unfollow(loser)

            except Exception, e:
                print >>sys.stderr, self.failWhale(e)

            else:
                self._printLoser(loser)


    def _printLoser(self, user):
        print "{u}".format(c=self.config, u=user)




class UnFollowLoader(cli.CommandFactory):
    implements(IPlugin)

    description = "Un-follow the given user"
    name = "unfollow"
    shortcut = "u"
    options = UnFollowOptions
    command = UnFollower


loader = UnFollowLoader()




import os, sys

from oauth import oauth

from twisted.internet.defer import Deferred, inlineCallbacks, returnValue
from twisted.plugin import IPlugin
from twisted.python.util import untilConcludes
from twisted.web.error import Error as WebError
from zope.interface import implements

from pendrell.agent import Agent
from jersey import log
from vtwt import cli


class OAuthOptions(cli.Options):
    optParameters = [
        ["consumer-key", "c", "xkCemphqejlUZ9W11MUMsA", "OAuth Consumer Key"],
        ["consumer-secret", "C", "QsnJx6SDhH6SjWNG95jcUR3FnKyFWFoOvDcf90E",
            "OAuth Consumer Secret"],
        ]


class OAuther(cli.CommandBase):

    BASE_URL = "https://api.twitter.com"
    TOKEN_REQUEST_URL = BASE_URL + "/oauth/request_token"
    AUTHORIZATION_URL = BASE_URL + "/oauth/authorize"
    ACCESS_TOKEN_URL = BASE_URL + "/oauth/access_token"

    @inlineCallbacks
    def execute(self):
        log.debug("Doing the OAuth")
        try:
            self.agent = Agent()
            self.consumer = self.buildConsumer()
            self.sig = oauth.OAuthSignatureMethod_HMAC_SHA1()

            self.requestToken = rt = yield self.getRequestToken()
            authUrl = self.getAuthorizationURL()
            print "Get authorization PIN from {0}".format(authUrl)
            pin = self._getPin()
            self.accessToken = at = yield self.getAccessToken(pin)
            print ("oauth_token = {0.key!r}\n"
                   "oauth_token_secret = {0.secret!r}").format(at)

        except Exception, e:
            print >>sys.stderr, self.failWhale(e)
            self.exitValue = os.EX_SOFTWARE


    def buildConsumer(self):
        return oauth.OAuthConsumer(
                self.config["consumer-key"],
                self.config["consumer-secret"])


    @inlineCallbacks
    def getRequestToken(self):
        tokenReq = oauth.OAuthRequest.from_consumer_and_token(self.consumer,
                http_url=self.TOKEN_REQUEST_URL)
        tokenReq.sign_request(self.sig, self.consumer, None)
        tokenRsp = yield self._openOAuth(tokenReq)
        token = oauth.OAuthToken.from_string(tokenRsp.content)
        returnValue(token)


    def getAuthorizationURL(self):
        authReq = oauth.OAuthRequest.from_token_and_callback(
                token=self.requestToken, http_url=self.AUTHORIZATION_URL)
        authReq.sign_request(self.sig, self.consumer, self.requestToken)
        return authReq.to_url()


    @inlineCallbacks
    def getAccessToken(self, pin):
        accessReq = oauth.OAuthRequest.from_consumer_and_token(self.consumer,
                token=self.requestToken, verifier=pin,
                http_url=self.ACCESS_TOKEN_URL)
        accessReq.sign_request(self.sig, self.consumer, self.requestToken)
        accessRsp = yield self._openOAuth(accessReq)
        token = oauth.OAuthToken.from_string(accessRsp.content)
        returnValue(token)


    def _openOAuth(self, req, **kw):
        kw.setdefault("method", req.http_method)
        kw.setdefault("headers", req.to_header())
        return self.agent.open(req.http_url, **kw)


    _pinPrompt = "PIN: "

    def _getPin(self):
        try:
            untilConcludes(sys.stdout.write, self._pinPrompt)
            untilConcludes(sys.stdout.flush)
            pin = raw_input()
        except (KeyboardInterrupt, OSError):
            log.err()
            raise NoOAuthPin()
        else:
            if pin:
                pin = pin.strip()
            if not pin:
                raise NoOAuthPin()
        return pin



class NoOAuthPin(Exception):
    def __init__(self):
        Exception.__init__(self, "No OAuth PIN")


class OAuthLoader(cli.CommandFactory):
    implements(IPlugin)

    description = "Get an OAuth Access Token"
    name = "oauth"
    shortcut = "o"
    options = OAuthOptions
    command = OAuther


loader = OAuthLoader()




import os, sys, traceback

from twisted.internet import reactor
from twisted.internet.defer import Deferred, inlineCallbacks, returnValue
from twisted.python.filepath import FilePath
from twisted.python.text import greedyWrap

from jersey import cli, log

from vtwt import util
from vtwt.svc import VtwtService

from oauth import oauth


# XXX HACK
from twisted.python import reflect
def safe_unicode(obj):
    return reflect._safeFormat(unicode, obj)
reflect.safe_str = safe_unicode


class Options(cli.Options):
    pass


class CommandBase(cli.Command):

    def __init__(self, config):
        cli.Command.__init__(self, config)


    def failWhale(self, error):
        config = self.config
        while config.parent and "COLUMNS" not in config:
            config = config.parent
        columns = config.get("COLUMNS", 80)
        return util.failWhale(error, columns)


    def formatMsgSimple(self, msg, screenNameWidth=None):
        fmt = "{msg.user.screen_name:{w}}  {text}"
        w = screenNameWidth if screenNameWidth else len(msg.user.screen_name)
        padding = " " * (w + 2)
        return self._formatMsg(msg, fmt, padding, w)


    def formatMsgLong(self, msg, screenNameWidth=None):
        fmt = "--- {msg.user.screen_name:{w}} {msg.created_at} [{msg.id}]\n" \
              "    {text}"
        w = screenNameWidth if screenNameWidth else len(msg.user.screen_name)
        padding = " " * 4
        return self._formatMsg(msg, fmt, padding, w)


    def _formatMsg(self, msg, fmt, padding, w):
        fmt = unicode(fmt)
        out = None
        while out is None:
            fmt = unicode(fmt)
            text = unicode(self._wrapText(msg, fmt, padding))
            out = fmt.format(msg=msg, text=text, w=w)
        return out


    def _wrapText(self, msg, fmt, padding):
        width = self.config.parent["COLUMNS"] - len(padding)
        lines = greedyWrap(msg.text, width)
        return ("\n"+padding).join(lines)


    def _print(self, text, stream=None):
        if stream is None:
            stream = sys.stdout
        enc = getattr(stream, "encoding", None)
        if enc:
            text.encode(enc)
        print >>stream, text



class Command(CommandBase):

    def __init__(self, config):
        CommandBase.__init__(self, config)
        self.vtwt = self._buildVtwt()

    def _buildVtwt(self):
        if "oauth-token" not in self.config.parent \
                or "oauth-token-secret" not in self.config.parent:
            raise cli.UsageError("No authentication token specified")
        oauthToken = oauth.OAuthToken(
                self.config.parent["oauth-token"],
                self.config.parent["oauth-token-secret"])
        svc = VtwtService(oauthToken)
        svc.setServiceParent(self)
        return svc



class CommandFactory(cli.CommandFactory):
    pass



class VtwtOptions(cli.PluggableOptions):

    defaultSubCommand = "watch"

    optFlags = [
            ["debug", "D", "Turn debugging messages on",]
        ]

    optParameters = [
            ["config-file", "c",
                os.path.expanduser("~/.vtwtrc"), "Vtwt config file",],
            ["oauth-token", "o", None, "OAuth access token."],
            ["oauth-token-secret", "O", None, "OAuth access token secret."],
        ]


    @property
    def commandPackage(self):
        import vtwt
        return vtwt


    def postOptions(self):
        if self["debug"]:
            self.logLevel = log.TRACE  # Allow all log messages.
        else:
            self.logLevel = log.ERROR+1  # Ignore ~all log messages.

        cf = FilePath(self["config-file"])
        if cf.exists():
            self.readConfigFile(cf)

        if self.subCommand != "oauth" and \
                not (self["oauth-token"] and self["oauth-token-secret"]):
            raise cli.UsageError(
                    "No OAuth token specified.  Run the 'oauth' subcommand.")

        self["COLUMNS"] = int(os.getenv("COLUMNS", 80))


    def readConfigFile(self, configFile):
        fileNS = dict()
        execfile(configFile.path, fileNS)
        for configKey in fileNS.iterkeys():
            k = configKey.replace("_", "-")
            if k in self and self[k] is None:
                self[k] = fileNS[configKey]



class VtwtCommander(cli.PluggableCommandRunner):

    def preApplication(self):
        # twittytwister.txml raises a weird Exception.  Suppress it.
        import logging
        logging.raiseExceptions = False



def run(args=sys.argv[:]):
    program = os.path.basename(args[0])
    args = args[1:]

    opts = VtwtOptions(program)
    try:
        opts.parseOptions()

        vtwt = VtwtCommander(program, opts)
        vtwt.run()

    except cli.UsageError, ue:
        print >>sys.stderr, str(opts)
        print >>sys.stderr, str(ue)
        raise SystemExit(os.EX_USAGE)

    else:
        raise SystemExit(vtwt.exitValue)





import sys

from twisted.internet.defer import inlineCallbacks, gatherResults
from twisted.plugin import IPlugin
from twisted.python import usage
from zope.interface import implements

from jersey import log
from vtwt import cli


class BlockOptions(cli.Options):

    def parseArgs(self, *names):
        if not names:
            raise usage.error("No one to block ;(")
        self["blockees"] = names


class Blocker(cli.Command):

    @inlineCallbacks
    def execute(self):
        for blockee in self.config["blockees"]:
            try:
                yield self._block(blockee)

            except Exception, e:
                print >>sys.stderr, self.failWhale(e)


    @inlineCallbacks
    def _block(self, user):
        yield self.vtwt.block(user)
        print "{0}".format(user)



class BlockLoader(cli.CommandFactory):
    implements(IPlugin)

    description = "Block the given user"
    name = "block"
    shortcut = "B"
    options = BlockOptions
    command = Blocker


loader = BlockLoader()




from twisted.internet.defer import inlineCallbacks, returnValue
from twisted.plugin import IPlugin
from zope.interface import implements

from jersey import log
from vtwt import cli


class FollowersOptions(cli.Options):
    optFlags = [
            ["long", "l", "Print more user information."],
        ]


class Followers(cli.Command):

    @inlineCallbacks
    def execute(self):
        try:
            followers = yield self.vtwt.getFollowers()

        except Exception, e:
            print >>sys.stderr, self.failWhale(e)

        else:
            self._printFollowers(followers)


    def _printFollowers(self, followers):
        for follower in followers:
            self._printFollower(follower)

    def _printFollower(self, follower):
        if self.config["long"]:
            followerFmt = "{0.screen_name} ({0.name})"
        else:
            followerFmt = "{0.screen_name}"

        print followerFmt.format(follower)



class FollowersLoader(cli.CommandFactory):
    implements(IPlugin)

    description = "List followers"
    name = "followers"
    shortcut = None
    options = FollowersOptions
    command = Followers


loader = FollowersLoader()




from twisted.internet.defer import Deferred, inlineCallbacks, returnValue
from twisted.plugin import IPlugin
from twisted.python.text import greedyWrap
from zope.interface import implements

from jersey import log
from vtwt import cli


class TweetOptions(cli.Options):

    def parseArgs(self, *tokens):
        self["tweet"] = " ".join(tokens)
        if not tokens:
            raise cli.UsageError("Nothing to tweet.")


class Tweeter(cli.Command):

    @inlineCallbacks
    def execute(self):
        try:
            text = self.config["tweet"]
            msgId = yield self.vtwt.tweet(text)
            wrapped = self._wrapText(text, len(str(msgId)))
            print "{0}  {1}".format(msgId, wrapped)

        except Exception, e:
            print >>sys.stderr, self.failWhale(e)



    def _wrapText(self, text, paddingLen):
        width = self.config.parent["COLUMNS"] - paddingLen
        joiner = "\n" + str(" " * paddingLen)
        return joiner.join(greedyWrap(text, width))



class TweetLoader(cli.CommandFactory):
    implements(IPlugin)

    description = "Tweet something"
    name = "tweet"
    shortcut = "t"
    options = TweetOptions
    command = Tweeter


loader = TweetLoader()




#!/usr/bin/env python

try:
    from setuptools import setup
except:
    from distutils.core import setup


def infoFromModule(path):
    ns = {}
    execfile(path, ns)
    return dict(
            name = ns["version"].package,
            version = ns["version"].short(),
            )


setup(
    packages = ["vtwt",],
    scripts = ["bin/vtwt",],

    description = "Ver's Twitter CLI",
    long_description = open("README").read(),

    url = "http://github.com/olix0r/vtwt",
    download_url = "http://pypi.python.org/pypi/vtwt",

    author = "Oliver Gould", author_email = "ver@olix0r.net",
    maintainer = "Oliver Gould", maintainer_email = "ver@olix0r.net",

    license = "MIT",
    classifiers = [
        "Framework :: Twisted",
        "License :: OSI Approved :: MIT License",
        ],

    provides = ["vtwt",],
    requires = [
        "twittytwister",
        "twisted",
        "oauth",
        "pendrell(>=0.2.2)",
        "jersey(>=0.1.2)",
        ],

    **infoFromModule("vtwt/_version.py")
    )


